local Grid2StatusAuraGroup = Grid2:NewModule("StatusAuraGroup")

--{{{  Default Setup

-- {{displaySpellId, <false = any>|<true = mine>|<blinkThreshold>, r, g, b[, r, g, b]...}, spellId1, spellId2, ...}
function Grid2:SetupDefaultAuraGroups(setup, class)
	local setupIndicator = setup.status
	if (class == "DRUID") then
		setup.auraGroups["group-GiftOfTheWild"] = {{26991, false, 1, 0, .8}, 26991, 26990}
		setup.auraGroups["group-Swiftmend"] = {{18562, false, 1, 0, .8}, 774, 8936}

		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-GiftOfTheWild", 39)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-Swiftmend", 49)
	elseif (class == "MAGE") then
		setup.auraGroups["group-DalaranBrilliance"] = {{61316, false, .1, .3, 1}, 61316, 27126, 27127, 61024}

		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-DalaranBrilliance", 99)
	elseif (class == "PALADIN") then
		setup.auraGroups["group-BlessingOfKings"] = {{25898, false, 1, 0, .8}, 20217, 25898}
		setup.auraGroups["group-BlessingOfMight"] = {{27141, false, 1, 0, .8}, 27140, 27141}
		setup.auraGroups["group-BlessingOfSanctuary"] = {{25899, false, 1, 0, .8}, 20911, 25899}
		setup.auraGroups["group-BlessingOfWisdom"] = {{27143, false, 1, 0, .8}, 27142, 27143}

		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-BlessingOfKings", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-BlessingOfMight", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-BlessingOfSanctuary", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-BlessingOfWisdom", 99)
	elseif (class == "PRIEST") then
		setup.auraGroups["group-ShadowProtection"] = {{39374, false, 1, 0, .8}, 25433, 39374}
		setup.auraGroups["group-DivineSpirit"] = {{32999, false, 1, 0, .8}, 27841, 32999}
		setup.auraGroups["group-PrayerOfFortitude"] = {{25392, false, 1, 0, .8}, 25389, 25392}

		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-ShadowProtection", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-DivineSpirit", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-PrayerOfFortitude", 99)
	end
end

local prev_MakeDefaultSetup = Grid2.MakeDefaultSetup
function Grid2:MakeDefaultSetup(...)
	local setup, class = prev_MakeDefaultSetup(self, ...)

	if (not setup.auraGroups) then
		setup.auraGroups = {}
		self:SetupDefaultAuraGroups(setup, class)
	end

	return setup, class
end

--}}}


--{{{  Status Creation

-- UpdateState for all Auras in the Group
local function status_UpdateState(self, unit, auraName, iconTexture, count, duration, expiration)
	if (self.auraNames[auraName]) then
		self.states[unit] = true
		self.textures[unit] = self.iconTexture
		self.counts[unit] = count
		self.durations[unit] = duration
		self.expirations[unit] = expiration
		self.new_state = self:IsActive(unit)
		self.new_count = count
	end
end

-- UpdateState for only my Auras in the Group
local function status_UpdateStateMine(self, unit, auraName, iconTexture, count, duration, expiration, isMine)
	if (self.auraNames[auraName] and isMine) then
		self.states[unit] = true
		self.textures[unit] = self.iconTexture
		self.counts[unit] = count
		self.durations[unit] = duration
		self.expirations[unit] = expiration
		self.new_state = self:IsActive(unit)
		self.new_count = count
	end
end

-- AuraGroup is implemented as a regular Aura status with modified detection code for UpdateState
function Grid2:SetupStatusAuraGroup(statusKey, infoAuraGroup)
	local info = infoAuraGroup[1]
	local displaySpellId = info[1]
	local status = self:CreateBuffStatus(unpack(info))
	status.name = statusKey -- force name

	local mine = info[2]
	status.UpdateState = mine and status_UpdateStateMine or status_UpdateState

	status.defaultDB.profile.displaySpellId = displaySpellId
	_, _, status.iconTexture = GetSpellInfo(displaySpellId)

	local auraNames = {}
	status.auraNames = auraNames
	for index = 2, # infoAuraGroup do
		local spellName = infoAuraGroup[index]
		if (type(spellName) == "number") then
			spellName = GetSpellInfo(spellName)
		end
		assert(type(spellName) == "string")
		auraNames[spellName] = true
	end

	self:RegisterStatus(status, { "color", "icon" })
	self:UpdateColorHandler(status)
	self:UpdateBlinkHandler(status)
	return status
end

-- Chain SetupAuraStatus to all StatusAuraGroup in the setup
local prev_SetupAuraStatus = Grid2.SetupAuraStatus
function Grid2:SetupAuraStatus(setup, ...)
	prev_SetupAuraStatus(self, setup, ...)

	for statusKey, infoAuraGroup in pairs(setup.auraGroups) do
		self:SetupStatusAuraGroup(statusKey, infoAuraGroup)
	end
end

--}}}


-- Chain the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(...)
	prev_LoadOptions(self, ...)
	if Grid2StatusAuraGroupOptions then return end
	if not IsAddOnLoaded("Grid2StatusAuraGroupOptions") then
		LoadAddOn("Grid2StatusAuraGroupOptions")
		if Grid2StatusAuraGroupOptions then
			Grid2StatusAuraGroupOptions:Initialize()
		end
	end
end
